import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

public class TicTacToeServer extends Frame {
  private TextArea output;
  private Player players[];
  private ServerSocket server;
  private int currentPlayer;
  private int NumPlayers = 2;
  public TicTacToeServer()
    {
      players = new Player[ NumPlayers ];
      currentPlayer = 0;
      // set up ServerSocket
      try {
	server = new ServerSocket( 4444, NumPlayers);
      }
      catch( IOException e ) {
	e.printStackTrace();
	System.exit( 1 );
      }
    }
  
  // wait for two connections so game can be played
  public void execute()
    {
      System.out.println("Server running, and ready to execute");
      for ( int i = 0; i < NumPlayers; i++ ) {
	try {
	  players[ i ] =
	    new Player( server.accept(), this, i );
	  players[ i ].start();
	}
	catch( IOException e ) {
	  e.printStackTrace();
	  System.exit( 1 );
	}
      }
      
				// Player X is suspended until Player O connects.
				// Resume player X now.
      players[ 0 ].resume();  
    }
   

  // Determine if a move is valid. 
  // This method is synchronized because only one move can be 
  // made at a time. 
  public synchronized boolean validMove( int loc, int player ) 
    { 
      boolean moveDone = false; 
 
      while ( player != currentPlayer ) { 
	try { 
	  wait(); 
	} 
	catch( InterruptedException e ) { 
	} 
      } 
      currentPlayer = ( currentPlayer + 1 ) % NumPlayers; 
				
      players[ currentPlayer ].otherPlayerMoved( loc ); 
      notify();    // tell waiting player to continue 
      return true; 
    } 
		
  public static void main( String args[] ) 
    { 
      TicTacToeServer game = new TicTacToeServer();
      game.execute(); 
    } 
} 
 
// Player class to manage each Player as a thread 
class Player extends Thread { 
  private Socket connection; 
  private DataInputStream input; 
  private DataOutputStream output; 
  private TicTacToeServer control; 
  private int number; 
  private int mark; 
  static int OPPONENT_MOVED = -1;
  static int PLAYER_CONNECTED = -2;
  static int WAITING = -3;
  static int BOTH_CONNECTED = -4;
  static int VALID_MOVE = -5;
  static int INVALID_MOVE = -6;
  public Player( Socket s, TicTacToeServer t, int num ) 
    { 
      mark =  num;  
 
      connection = s; 
       
      try { 
	input = new DataInputStream( 
				    connection.getInputStream() ); 
	output = new DataOutputStream( 
				      connection.getOutputStream() ); 
      } 
      catch( IOException e ) { 
	e.printStackTrace(); 
	System.exit( 1 ); 
      } 
 
      control = t; 
      number = num; 
    } 
 
  public void otherPlayerMoved( int loc ) 
    { 
      try { 
	output.writeInt(OPPONENT_MOVED);
	output.writeInt( loc ); 
      } 
      catch ( IOException e ) { e.printStackTrace(); } 
    } 
 
  public void run() 
    { 
      boolean done = false; 
 
      try {  
	output.writeInt( mark ); 
	// wait for another player to arrive 
	if ( mark == 0 ) {
	  output.writeInt(WAITING);
	  suspend();  // suspend until player O connects
	  output.writeInt(BOTH_CONNECTED);
	}

	// Play game
	while ( !done ) {
	  int location = input.readInt();
	  control.validMove( location, number );  
	  output.writeInt(VALID_MOVE);
	}         
	connection.close();
      }
      catch( IOException e ) {
	e.printStackTrace();
	System.exit( 1 );
      }
    }
}
